import 'package:osam/osam.dart';
import 'package:osam_flutter/src/navigation/osam_route.dart';

import '../../osam_flutter.dart';

part 'navigation.g.dart';

class NoRouteException implements Exception {}

@HiveType(typeId: 221)
class Navigation {
  @HiveField(0)
  final PropertyNotifier<List<OsamRoute<Object, Object>>> routes;

  Navigation(this.routes);

  Future<R> push<A, R>(OsamRoute<A, R> route) {
    routes.update((value) => value.add(route));
    return route.whenComplete;
  }

  void pop() => routes.update((value) => value.removeLast());

  OsamRoute<Object, Object> findRoute(String route) =>
      routes.value.firstWhere((element) => element.value == route);

  Future<R> pushReplacement<A, R>(OsamRoute<A, R> route) {
    routes.update((value) {
      value.removeLast();
      value.add(route);
    });
    return route.whenComplete;
  }

  Future<R> pushAndRemoveUntil<A, R>(OsamRoute<A, R> route) {
    routes.update((value) {
      value.clear();
      value.add(route);
    });
    return route.whenComplete;
  }

  void popUntil(String route) => routes.update((value) {
        if (value.contains(route)) {
          while (value.last.value != route) {
            value.removeLast();
          }
        }
      });
}

@HiveType(typeId: 220)
class TabNavigation {
  @HiveField(0)
  final PropertyNotifier<String> tabNotifier;

  TabNavigation(this.tabNotifier);

  set tab(String tab) => tabNotifier.value = tab;
}
