import 'package:osam_flutter/osam_flutter.dart';
import 'package:osam_flutter/src/navigation/osam_route.dart';

mixin NavigationPresenter<US, AS> on Presenter<US, AS> {
  Navigation get navigation;

  void pop() => navigation.pop();

  void popUntil(String route) => navigation.popUntil(route);

  void setRouteResult<R>({String route, R result}) => findRoute(route).result = result;

  Future<R> push<A, R>(OsamRoute<A, R> route) => navigation.push(route);

  Future<R> pushReplacement<A, R>(OsamRoute<A, R> route) => navigation.pushReplacement(route);

  Future<R> pushAndRemoveUntil<A, R>(OsamRoute<A, R> route) => navigation.pushAndRemoveUntil(route);

  List<OsamRoute<Object, Object>> get currentRoutes => navigation.routes.value;

  OsamRoute<Object, Object> get currentRoute => currentRoutes.last;

  OsamRoute<Object, Object> findRoute(String route) =>
      isInStack(route) ? navigation.findRoute(route) : null;

  OsamRoute<Object, Object> get firstRoute => currentRoutes.first;

  bool isInStack(String route) => currentRoutes.contains(route);

  bool get canPop => navigation.routes.value.length > 1;

  OsamRoute<Object, Object> get routeBefore =>
      currentRoutes.length > 1 ? currentRoutes[currentRoutes.length - 2] : throw null;
}

mixin TabNavigationPresenter<US, AS> on Presenter<US, AS> {
  TabNavigation get tabNavigation;

  String get currentTab => tabNavigation.tabNotifier.value;

  void changeTab(String newTab) => tabNavigation.tab = newTab;
}
